/*
 * "Hello World" example.
 *
 * This example prints 'Hello from Nios II' to the STDOUT stream. It runs on
 * the Nios II 'standard', 'full_featured', 'fast', and 'low_cost' example
 * designs. It runs with or without the MicroC/OS-II RTOS and requires a STDOUT
 * device in your system's hardware.
 * The memory footprint of this hosted application is ~69 kbytes by default
 * using the standard reference design.
 *
 * For a reduced footprint version of this template, and an explanation of how
 * to reduce the memory footprint for a given application, see the
 * "small_hello_world" template.
 *
 */

#include <stdio.h>
#include <stdlib.h>
#include <stddef.h>
#include <string.h>
#include <math.h>
#include "sys/alt_cache.h"
#include "sys/alt_alarm.h"
#include "system.h"
#include "sys/alt_irq.h"
#include <io.h>
#include "altera_avalon_timer_regs.h"
#include "altera_avalon_uart_regs.h"

#include "LedSampleApl.h"
#include "board_init.h"

typedef struct _led_apl_sample_type_ {
  LED_APL_TYPE type;
  unsigned char init_data;
} LED_APL_SAMPLE_TYPE;

LED_APL_SAMPLE_TYPE led_apl_sample[] = {
  {LED_TYPE_ALLOFF,		LED_ALLOFF_INIT_DAT  },	/* S */
  {LED_TYPE_ALLON,		LED_ALLON_INIT_DAT 	 },	/* S_ */
  {LED_TYPE_ALLBLINK,	LED_ALLBLINK_INIT_DAT},	/* S_ */
  {LED_TYPE_BLINK,		LED_BLINK_INIT_DAT   }, /* ݓ_ */
  {LED_TYPE_SHIFT,		LED_SHIFT_INIT_DAT	 }, /* Vtg */
  {LED_TYPE_INC,		LED_INC_INIT_DAT     },	/* CNg */
  {LED_TYPE_DEC,		LED_DEC_INIT_DAT     },	/* fNg */
  {LED_TYPE_FREE,		0x77                 },	/* Cӓ_ */
  {LED_TYPE_FREE,		0x88                 }	/* Cӓ_ */
};

int now_led_apl_type = LED_TYPE_ALLBLINK;

static void led_apl_chg_type (void);

int main()
{
  board_init ();
  //printf("Hello from Nios II! test Sample\n");


  LedApl_Init ();

  /* ݒ */
  LedApl_Set_Type (led_apl_sample[now_led_apl_type].type, 
				   led_apl_sample[now_led_apl_type].init_data);
  timer_start (TIMER_1_NAME);

  while (1) 
	{
	  LedApl_Exec ();
	  led_apl_chg_type ();
	}

  return 0;
}

static void led_apl_chg_type (void)
{
  static int key_in = 0;

  if (key_in == 0)
	{
	  if (pio_input (PD4) == 1)
		{
		  /* Key ꂽ̂Ł@TYPEύX */
		  now_led_apl_type++;
		  if (now_led_apl_type >= sizeof(led_apl_sample)/sizeof(led_apl_sample[0]))
			{
			  now_led_apl_type = LED_TYPE_ALLOFF;
			}

		  LedApl_Set_Type (led_apl_sample[now_led_apl_type].type, 
						   led_apl_sample[now_led_apl_type].init_data);
		  key_in = 1;
		}
	}
  else
	{
	  if (pio_input (PD4) != 1)
		{
		  key_in = 0;
		}
	}
  return;
}

void timer0_irq_event(void* context, alt_u32 id)
{
	/* IRQ Handler TIMER0 disable */
	irq_disable( &TIMER_board_info[0].irq );

	/* TIMER0 interrupt clear */
	TIMER_write( &TIMER_board_info[0], TIMER_STR, 0 );

	/* Interrupt processing */

	/* IRQ Handler TIMER0 enavle */
	irq_enable( &TIMER_board_info[0].irq );
	/* TIMER0 count start */
	timer_start( TIMER_0_NAME );

}

/* Interval Timer gꍇ̃R[obN֐ */
void timer1_irq_event(void* context, alt_u32 id)
{
	/* IRQ Handler TIMER1 disable */
	irq_disable( &TIMER_board_info[1].irq );

	/* TIMER1 interrupt clear */
	TIMER_write( &TIMER_board_info[1], TIMER_STR, 0 );

	/* Interrupt processing */
	 LedApl_Irq_Timer ();

	/* IRQ Handler TIMER1 enavle */
	irq_enable( &TIMER_board_info[1].irq );
	/* TIMER1 count start */
	timer_start( TIMER_1_NAME );
}

void timer2_irq_event(void* context, alt_u32 id)
{
	/* IRQ Handler TIMER2 disable */
	irq_disable( &TIMER_board_info[2].irq );

	/* TIMER2 interrupt clear */
	TIMER_write( &TIMER_board_info[2], TIMER_STR, 0 );

	/* Interrupt processing */

	/* IRQ Handler TIMER2 enavle */
	irq_enable( &TIMER_board_info[2].irq );
	/* TIMER2 count start */
	timer_start( TIMER_2_NAME );

}

void uart_cmd_event(void* context)
{
  return;
}

void eint0_irq_event( void* context, alt_u32 id)
{

	/* IRQ Handler EINT0 disable */
	irq_disable( &EINT_board_info.dev_info[EINT0].irq );

	/* Interrupt processing */


	/* IRQ Handler EINT0 enable */
	irq_enable( &EINT_board_info.dev_info[EINT0].irq );

}

void eint1_irq_event( void* context, alt_u32 id)
{

	/* IRQ Handler EINT1 disable */
	irq_disable( &EINT_board_info.dev_info[EINT1].irq );

	/* Interrupt processing */


	/* IRQ Handler EINT1 enable */
	irq_enable( &EINT_board_info.dev_info[EINT1].irq );

}

void eint2_irq_event( void* context, alt_u32 id)
{

	/* IRQ Handler EINT2 disable */
	irq_disable( &EINT_board_info.dev_info[EINT2].irq );

	/* Interrupt processing */


	/* IRQ Handler EINT2 enable */
	irq_enable( &EINT_board_info.dev_info[EINT2].irq );

}

void eint3_irq_event( void* context, alt_u32 id)
{

	/* IRQ Handler EINT3 disable */
	irq_disable( &EINT_board_info.dev_info[EINT3].irq );

	/* Interrupt processing */


	/* IRQ Handler EINT3 enable */
	irq_enable( &EINT_board_info.dev_info[EINT3].irq );

}

void eint4_irq_event( void* context, alt_u32 id)
{
	/* IRQ Handler EINT4 disable */
	irq_disable( &EINT_board_info.dev_info[EINT4].irq );

	/* Interrupt processing */

	/* IRQ Handler EINT5 enable */
	irq_enable( &EINT_board_info.dev_info[EINT4].irq );

}

void eint5_irq_event( void* context, alt_u32 id)
{

	/* IRQ Handler EINT5 disable */
	irq_disable( &EINT_board_info.dev_info[EINT5].irq );

	/* Interrupt processing */


	/* IRQ Handler EINT5 enable */
	irq_enable( &EINT_board_info.dev_info[EINT5].irq );

}

void eint6_irq_event( void* context, alt_u32 id)
{

	/* IRQ Handler EINT6 disable */
	irq_disable( &EINT_board_info.dev_info[EINT6].irq );

	/* Interrupt processing */


	/* IRQ Handler EINT6 enable */
	irq_enable( &EINT_board_info.dev_info[EINT6].irq );

}

void eint7_irq_event( void* context, alt_u32 id)
{

	/* IRQ Handler EINT7 disable */
	irq_disable( &EINT_board_info.dev_info[EINT7].irq );

	/* Interrupt processing */


	/* IRQ Handler EINT7 enable */
	irq_enable( &EINT_board_info.dev_info[EINT7].irq );

}
